"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBrowserHydrationEnvService = exports.setDefaultHydratingEnvService = exports.hasDefaultHydrationEnvService = exports.getDefaultHydrationEnvService = void 0;
const retil_mount_1 = require("retil-mount");
const retil_source_1 = require("retil-source");
const retil_support_1 = require("retil-support");
// Servers don't do hydrationg, so we don't modify the env at all.
const serverHydratingEnvSource = retil_source_1.constant(retil_mount_1.createEnvVector([{}]));
const serverHydrationEnvService = [
    serverHydratingEnvSource,
    retil_support_1.noop,
];
const getDefaultHydrationEnvService = () => {
    if (!exports.getDefaultHydrationEnvService.value) {
        const isBrowser = typeof window !== 'undefined';
        exports.getDefaultHydrationEnvService.value = isBrowser
            ? createBrowserHydrationEnvService()
            : serverHydrationEnvService;
    }
    return exports.getDefaultHydrationEnvService.value;
};
exports.getDefaultHydrationEnvService = getDefaultHydrationEnvService;
const hasDefaultHydrationEnvService = () => {
    return !!exports.getDefaultHydrationEnvService.value;
};
exports.hasDefaultHydrationEnvService = hasDefaultHydrationEnvService;
const setDefaultHydratingEnvService = (value) => {
    exports.getDefaultHydrationEnvService.value = value;
};
exports.setDefaultHydratingEnvService = setDefaultHydratingEnvService;
function createBrowserHydrationEnvService(options = {}) {
    const { default: defaultHydrationService, disablePrecache } = options;
    const hasDefault = exports.hasDefaultHydrationEnvService();
    if (hasDefault && defaultHydrationService) {
        throw new Error('Could not override the default hydration service.');
    }
    const [hydratingSource, setHydrating] = retil_source_1.createState(retil_mount_1.createEnvVector(disablePrecache ? [true] : [true, false]));
    const hydratingEnvSource = retil_mount_1.fuseEnvSource((use) => ({
        hydrating: use(hydratingSource),
    }));
    let hasHydrated = false;
    const hydrate = () => {
        if (!hasHydrated) {
            hasHydrated = true;
            setHydrating(retil_mount_1.createEnvVector([false]));
        }
    };
    const service = [hydratingEnvSource, hydrate];
    if (!hasDefault &&
        (defaultHydrationService || defaultHydrationService === undefined)) {
        exports.setDefaultHydratingEnvService(service);
    }
    return service;
}
exports.createBrowserHydrationEnvService = createBrowserHydrationEnvService;
